==> 0.0.6.tar.gz: lib/manager/v2_tim_message_manager.dart <==
import 'dart:convert';

import 'package:flutter/material.dart';

import 'package:flutter/services.dart';
import 'package:tencent_im_sdk_plugin/models/v2_tim_callback.dart';
import 'package:tencent_im_sdk_plugin/models/v2_tim_message.dart';
import 'package:tencent_im_sdk_plugin/models/v2_tim_value_callback.dart';

///提供高级消息处理相关接口
///
/// [addAdvancedMsgListener] 添加高级消息的事件监听器
///
/// [removeAdvancedMsgListener] 移除高级消息监听器
///
/// [createTextMessage] 创建文本消息
///
/// [createTextAtMessage] 创建文本消息，并且可以附带 @ 提醒功能
///
/// [createCustomMessage] 创建自定义消息
///
/// [createImageMessage] 创建图片消息（图片最大支持 28 MB）
///
/// [createSoundMessage] 创建语音消息（语音最大支持 28 MB）
///
/// [createVideoMessage] 创建视频消息（视频最大支持 100 MB）
///
/// [createVideoMessage] 创建文件消息（文件最大支持 100 MB）
///
/// [createLocationMessage] 创建地理位置消息
///
/// [createFaceMessage] 创建表情消息
///
/// [sendMessage] 发送高级消息（高级版本：可以指定优先级，推送信息等特性）
///
/// [getC2CHistoryMessageList] 获取单聊历史消息
///
/// [getGroupHistoryMessageList] 获取群组历史消息
///
/// [revokeMessage] 撤回消息
///
/// [markC2CMessageAsRead] 设置单聊消息已读
///
/// [markGroupMessageAsRead] 设置群组消息已读
///
/// [deleteMessageFromLocalStorage] 删除本地消息
///
/// [deleteMessages] 删除本地及漫游消息
///
///{@category Manager}
///
class V2TIMMessageManager {
  ///@nodoc
  MethodChannel _channel = null;

  ///@nodoc
  Set advancedMsgListener = new Set();

  ///@nodoc
  V2TIMMessageManager(channel) {
    this._channel = channel;
  }

  /// 添加高级消息的事件监听器
  ///
  void addAdvancedMsgListener(Function listener) {
    advancedMsgListener.add(listener);
    _channel.invokeMethod("addAdvancedMsgListener", _buildParam({}));
  }

  /// 移除高级消息监听器
  ///
  void removeAdvancedMsgListener(Function listener) {
    advancedMsgListener.clear();
    _channel.invokeMethod("removeAdvancedMsgListener", _buildParam({}));
  }

  /// 创建文本消息
  ///
  Future<String> createTextMessage({
    @required String text,
  }) async {
    return await _channel.invokeMethod(
      "createTextMessage",
      _buildParam(
        {
          "text": text,
        },
      ),
    );
  }

  /// 创建文本消息，并且可以附带 @ 提醒功能
  ///
  /// ```
  /// 提醒消息仅适用于在群组中发送的消息
  /// ```
  ///
  /// 参数
  ///
  /// ```
  /// userList	需要 @ 的用户列表，如果需要 @ALL，请传入 AT_ALL_TAG 常量字符串。 举个例子，假设该条文本消息希望@提醒 denny 和 lucy 两个用户，同时又希望@所有人，atUserList 传 ["denny","lucy",AT_ALL_TAG]
  /// ```
  ///
  Future<String> createTextAtMessage({
    @required String text,
    @required List<String> userList,
  }) async {
    return await _channel.invokeMethod(
      "createTextAtMessage",
      _buildParam(
        {
          "text": text,
          "userList": userList,
        },
      ),
    );
  }

  /// 创建自定义消息
  ///
  Future<String> createCustomMessage({
    @required String data,
  }) async {
    return await _channel.invokeMethod(
      "createCustomMessage",
      _buildParam(
        {
          "data": data,
        },
      ),
    );
  }

  /// 创建图片消息（图片最大支持 28 MB）
  ///
  Future<String> createImageMessage({
    @required String imagePath,
  }) async {
    return await _channel.invokeMethod(
      "createImageMessage",
      _buildParam(
        {
          "imagePath": imagePath,
        },
      ),
    );
  }

  /// 创建语音消息（语音最大支持 28 MB）
  ///
  /// 参数
  ///
  /// ```
  /// duration	语音时长，单位 s
  /// ```
  ///
  Future<String> createSoundMessage({
    @required String soundPath,
    @required int duration,
  }) async {
    return await _channel.invokeMethod(
      "createSoundMessage",
      _buildParam(
        {
          "soundPath": soundPath,
          "duration": duration,
        },
      ),
    );
  }

  /// 创建视频消息（视频最大支持 100 MB）
  ///
  /// 参数
  ///
  /// ```
  /// type	视频类型，如 mp4 mov 等
  /// duration	视频时长，单位 s
  /// snapshotPath	视频封面图片路径
  /// ```
  ///
  Future<String> createVideoMessage({
    @required String videoFilePath,
    @required String type,
    @required int duration,
    @required String snapshotPath,
  }) async {
    return await _channel.invokeMethod(
      "createVideoMessage",
      _buildParam(
        {
          "videoFilePath": videoFilePath,
          "type": type,
        },
      ),
    );
  }

  /// 创建文件消息（文件最大支持 100 MB）
  ///
  Future<String> createFileMessage({
    @required String filePath,
    @required String fileName,
  }) async {
    return await _channel.invokeMethod(
      "createFileMessage",
      _buildParam(
        {
          "filePath": filePath,
          "fileName": fileName,
        },
      ),
    );
  }

  /// 创建地理位置消息
  ///
  Future<String> createLocationMessage({
    @required String desc,
    @required double longitude,
    @required double latitude,
  }) async {
    return await _channel.invokeMethod(
      "createLocationMessage",
      _buildParam(
        {
          "desc": desc,
          "longitude": longitude,
          "latitude": latitude,
        },
      ),
    );
  }

  /// 创建表情消息
  ///
  /// ```
  /// SDK 并不提供表情包，如果开发者有表情包，可使用 index 存储表情在表情包中的索引， 或者直接使用 data 存储表情二进制信息以及字符串 key，都由用户自定义，SDK 内部只做透传。
  /// ```
  ///
  /// 参数
  ///
  /// ```
  /// index	表情索引
  /// data	自定义数据
  /// ```
  ///
  Future<String> createFaceMessage({
    @required int index,
    @required String data,
  }) async {
    return await _channel.invokeMethod(
      "createFaceMessage",
      _buildParam(
        {
          "index": index,
          "data": data,
        },
      ),
    );
  }

  /// 获取单聊历史消息
  ///
  /// 参数
  ///
  /// ```
  /// count	拉取消息的个数，不宜太多，会影响消息拉取的速度，这里建议一次拉取 20 个
  /// lastMsg	获取消息的起始消息，如果传 null，起始消息为会话的最新消息
  /// ```
  ///
  /// 注意
  ///
  /// ```
  /// 如果 SDK 检测到没有网络，默认会直接返回本地数据
  /// ```
  ///
  Future<V2TimValueCallback<List<V2TimMessage>>> getC2CHistoryMessageList({
    @required String userID,
    @required int count,
    String lastMsgID,
  }) async {
    return V2TimValueCallback<List<V2TimMessage>>.fromJson(
      _formatJson(
        await _channel.invokeMethod(
          "getC2CHistoryMessageList",
          _buildParam(
            {
              "userID": userID,
              "count": count,
              "lastMsgID": lastMsgID,
            },
          ),
        ),
      ),
    );
  }

  /// 获取群组历史消息
  ///
  /// 参数
  ///
  /// ```
  /// count	拉取消息的个数，不宜太多，会影响消息拉取的速度，这里建议一次拉取 20 个
  /// lastMsg	获取消息的起始消息，如果传 null，起始消息为会话的最新消息
  /// ```
  ///
  /// 注意
  ///
  /// ```
  /// 如果 SDK 检测到没有网络，默认会直接返回本地数据
  /// 只有会议群（Meeting）才能拉取到进群前的历史消息，直播群（AVChatRoom）消息不存漫游和本地数据库，调用这个接口无效
  /// ```
  ///
  Future<V2TimValueCallback<List<V2TimMessage>>> getGroupHistoryMessageList({
    @required String groupID,
    @required int count,
    String lastMsgID,
  }) async {
    return V2TimValueCallback<List<V2TimMessage>>.fromJson(
      _formatJson(
        await _channel.invokeMethod(
          "getGroupHistoryMessageList",
          _buildParam(
            {
              "groupID": groupID,
              "count": count,
              "lastMsgID": lastMsgID,
            },
          ),
        ),
      ),
    );
  }

  /// 撤回消息
  ///
  /// 注意
  ///
  /// ```
  /// 撤回消息的时间限制默认 2 minutes，超过 2 minutes 的消息不能撤回，您也可以在 控制台（功能配置 -> 登录与消息 -> 消息撤回设置）自定义撤回时间限制。
  /// 仅支持单聊和群组中发送的普通消息，无法撤销 onlineUserOnly 为 true 即仅在线用户才能收到的消息，也无法撤销直播群（AVChatRoom）中的消息。
  /// 如果发送方撤回消息，已经收到消息的一方会收到 V2TIMAdvancedMsgListener -> onRecvMessageRevoked 回调。
  /// ```
  ///
  Future<V2TimCallback> revokeMessage({
    @required String msgID,
  }) async {
    return V2TimCallback.fromJson(
      _formatJson(
        await _channel.invokeMethod(
          "revokeMessage",
          _buildParam(
            {
              "msgID": msgID,
            },
          ),
        ),
      ),
    );
  }

  ///设置单聊消息已读
  ///
  Future<V2TimCallback> markC2CMessageAsRead({
    @required String userID,
  }) async {
    return V2TimCallback.fromJson(
      _formatJson(
        await _channel.invokeMethod(
          "markC2CMessageAsRead",
          _buildParam(
            {
              "userID": userID,
            },
          ),
        ),
      ),
    );
  }

  /// 设置群组消息已读
  ///
  Future<V2TimCallback> markGroupMessageAsRead({
    @required String groupID,
  }) async {
    return V2TimCallback.fromJson(
      _formatJson(
        await _channel.invokeMethod(
          "markGroupMessageAsRead",
          _buildParam(
            {
              "groupID": groupID,
            },
          ),
        ),
      ),
    );
  }

  /// 删除本地消息
  ///
  /// 注意
  ///
  /// ```
  /// 消息只能本地删除，消息删除后，SDK 会在本地把这条消息标记为已删除状态，getHistoryMessage 不能再拉取到， 但是如果程序卸载重装，本地会失去对这条消息的删除标记，getHistoryMessage 还能再拉取到该条消息。
  /// ```
  ///
  Future<V2TimCallback> deleteMessageFromLocalStorage({
    @required String msgID,
  }) async {
    return V2TimCallback.fromJson(
      _formatJson(
        await _channel.invokeMethod(
          "deleteMessageFromLocalStorage",
          _buildParam(
            {
              "msgID": msgID,
            },
          ),
        ),
      ),
    );
  }

  /// 删除本地及漫游消息
  ///
  /// 注意
  ///
  /// ```
  ///该接口会删除本地历史的同时也会把漫游消息即保存在服务器上的消息也删除，卸载重装后无法再拉取到。需要注意的是：
  ///   一次最多只能删除 30 条消息
  ///   要删除的消息必须属于同一会话
  ///   一秒钟最多只能调用一次该接口
  ///   如果该账号在其他设备上拉取过这些消息，那么调用该接口删除后，这些消息仍然会保存在那些设备上，即删除消息不支持多端同步。
  /// ```
  ///
  Future<V2TimCallback> deleteMessages({
    @required List<String> msgIDs,
  }) async {
    return V2TimCallback.fromJson(
      _formatJson(
        await _channel.invokeMethod(
          "deleteMessages",
          _buildParam(
            {
              "msgIDs": msgIDs,
            },
          ),
        ),
      ),
    );
  }

  Future<V2TimValueCallback<V2TimMessage>> insertGroupMessageToLocalStorage({
    @required String msgID,
    String groupID,
    @required String sender,
  }) async {
    return V2TimValueCallback<V2TimMessage>.fromJson(
      _formatJson(
        await _channel.invokeMethod(
          "insertGroupMessageToLocalStorage",
          _buildParam(
            {
              "msgID": msgID,
              "groupID": groupID,
              "sender": sender,
            },
          ),
        ),
      ),
    );
  }

  /// 发送高级消息（高级版本：可以指定优先级，推送信息等特性）
  ///
  /// 参数
  ///
  /// ```
  /// msg	待发送的消息对象，需要通过对应的 createXXXMessage 接口进行创建。
  /// receiver	消息接收者的 userID, 如果是发送 C2C 单聊消息，只需要指定 receiver 即可。
  /// groupID	目标群组 ID，如果是发送群聊消息，只需要指定 groupID 即可。
  /// priority	消息优先级，仅针对群聊消息有效。请把重要消息设置为高优先级（比如红包、礼物消息），高频且不重要的消息设置为低优先级（比如点赞消息）。
  /// onlineUserOnly	是否只有在线用户才能收到，如果设置为 true ，接收方历史消息拉取不到，常被用于实现“对方正在输入”或群组里的非重要提示等弱提示功能。
  /// offlinePushInfo	离线推送时携带的标题和内容。
  /// ```
  ///
  /// 返回
  ///
  /// ```
  /// 消息唯一标识
  /// ```
  ///
  /// 注意
  ///
  /// ```
  /// 设置 offlinePushInfo 字段，需要先在 V2TIMOfflinePushManager 开启推送。推送开启后，除了自定义消息，其他消息默认都会推送。
  /// 自定义消息默认不会推送，如果需要推送，请设置 offlinePushInfo 的 desc 字段，推送的时候会默认展示 desc 信息。
  /// ```
  ///
  Future<V2TimValueCallback<V2TimMessage>> sendMessage({
    String msgID,
    String receiver,
    String groupID,
    int priority,
    bool onlineUserOnly,
    Map offlinePushInfo,
  }) async {
    return V2TimValueCallback<V2TimMessage>.fromJson(
      _formatJson(
        await _channel.invokeMethod(
          "sendMessage",
          _buildParam(
            {
              "msgID": msgID,
              "groupID": groupID,
              "receiver": receiver,
              "priority": priority,
              "onlineUserOnly": onlineUserOnly,
              "offlinePushInfo": offlinePushInfo
            },
          ),
        ),
      ),
    );
  }

  ///@nodoc
  Map _buildParam(Map param) {
    param["TIMManagerName"] = "messageManager";
    return param;
  }

  ///@nodoc
  _formatJson(jsonSrc) {
    // print('json');
    // print(json.encode(jsonSrc));
    return json.decode(json.encode(jsonSrc));
  }
}

==> 0.0.10.tar.gz: lib/manager/v2_tim_message_manager.dart <==
import 'dart:convert';

import 'package:flutter/material.dart';

import 'package:flutter/services.dart';
import 'package:tencent_im_sdk_plugin/models/v2_tim_callback.dart';
import 'package:tencent_im_sdk_plugin/models/v2_tim_message.dart';
import 'package:tencent_im_sdk_plugin/models/v2_tim_value_callback.dart';

///提供高级消息处理相关接口
///
/// [addAdvancedMsgListener] 添加高级消息的事件监听器
///
/// [removeAdvancedMsgListener] 移除高级消息监听器
///
/// [createTextMessage] 创建文本消息
///
/// [createTextAtMessage] 创建文本消息，并且可以附带 @ 提醒功能
///
/// [createCustomMessage] 创建自定义消息
///
/// [createImageMessage] 创建图片消息（图片最大支持 28 MB）
///
/// [createSoundMessage] 创建语音消息（语音最大支持 28 MB）
///
/// [createVideoMessage] 创建视频消息（视频最大支持 100 MB）
///
/// [createVideoMessage] 创建文件消息（文件最大支持 100 MB）
///
/// [createLocationMessage] 创建地理位置消息
///
/// [createFaceMessage] 创建表情消息
///
/// [sendMessage] 发送高级消息（高级版本：可以指定优先级，推送信息等特性）
///
/// [getC2CHistoryMessageList] 获取单聊历史消息
///
/// [getGroupHistoryMessageList] 获取群组历史消息
///
/// [revokeMessage] 撤回消息
///
/// [markC2CMessageAsRead] 设置单聊消息已读
///
/// [markGroupMessageAsRead] 设置群组消息已读
///
/// [deleteMessageFromLocalStorage] 删除本地消息
///
/// [deleteMessages] 删除本地及漫游消息
///
///{@category Manager}
///
class V2TIMMessageManager {
  ///@nodoc
  MethodChannel _channel = null;

  ///@nodoc
  Set advancedMsgListener = new Set();

  ///@nodoc
  V2TIMMessageManager(channel) {
    this._channel = channel;
  }

  /// 添加高级消息的事件监听器
  ///
  void addAdvancedMsgListener(Function listener) {
    advancedMsgListener.add(listener);
    _channel.invokeMethod("addAdvancedMsgListener", _buildParam({}));
  }

  /// 移除高级消息监听器
  ///
  void removeAdvancedMsgListener(Function listener) {
    advancedMsgListener.clear();
    _channel.invokeMethod("removeAdvancedMsgListener", _buildParam({}));
  }

  ///发送图片消息
  ///
  Future<V2TimValueCallback<V2TimMessage>> sendImageMessage(
      {@required String imagePath,
      @required String receiver,
      String groupID,
      int priority,
      bool onlineUserOnly}) async {
    return V2TimValueCallback<V2TimMessage>.fromJson(
      _formatJson(
        await _channel.invokeMethod(
          "sendImageMessage",
          _buildParam(
            {
              "imagePath": imagePath,
              "receiver": receiver,
              "groupID": groupID,
              "priority": priority,
              "onlineUserOnly": onlineUserOnly,
            },
          ),
        ),
      ),
    );
  }

  /// 创建文本消息
  ///
  Future<V2TimValueCallback<String>> createTextMessage({
    @required String text,
  }) async {
    return V2TimValueCallback<String>.fromJson(
      _formatJson(
        await _channel.invokeMethod(
          "createTextMessage",
          _buildParam(
            {
              "text": text,
            },
          ),
        ),
      ),
    );
  }

  /// 创建文本消息，并且可以附带 @ 提醒功能
  ///
  /// ```
  /// 提醒消息仅适用于在群组中发送的消息
  /// ```
  ///
  /// 参数
  ///
  /// ```
  /// userList	需要 @ 的用户列表，如果需要 @ALL，请传入 AT_ALL_TAG 常量字符串。 举个例子，假设该条文本消息希望@提醒 denny 和 lucy 两个用户，同时又希望@所有人，atUserList 传 ["denny","lucy",AT_ALL_TAG]
  /// ```
  ///
  Future<V2TimValueCallback<String>> createTextAtMessage({
    @required String text,
    @required List<String> userList,
  }) async {
    return V2TimValueCallback<String>.fromJson(
      _formatJson(
        await _channel.invokeMethod(
          "createTextAtMessage",
          _buildParam(
            {
              "text": text,
              "userList": userList,
            },
          ),
        ),
      ),
    );
  }

  /// 创建自定义消息
  ///
  Future<V2TimValueCallback<String>> createCustomMessage({
    @required String data,
  }) async {
    return V2TimValueCallback<String>.fromJson(
      _formatJson(
        await _channel.invokeMethod(
          "createCustomMessage",
          _buildParam(
            {
              "data": data,
            },
          ),
        ),
      ),
    );
  }

  /// 创建图片消息（图片最大支持 28 MB）
  ///
  Future<V2TimValueCallback<String>> createImageMessage({
    @required String imagePath,
  }) async {
    return V2TimValueCallback<String>.fromJson(
      _formatJson(
        await _channel.invokeMethod(
          "createImageMessage",
          _buildParam(
            {
              "imagePath": imagePath,
            },
          ),
        ),
      ),
    );
  }

  /// 创建语音消息（语音最大支持 28 MB）
  ///
  /// 参数
  ///
  /// ```
  /// duration	语音时长，单位 s
  /// ```
  ///
  Future<V2TimValueCallback<String>> createSoundMessage({
    @required String soundPath,
    @required int duration,
  }) async {
    return V2TimValueCallback<String>.fromJson(
      _formatJson(
        await _channel.invokeMethod(
          "createSoundMessage",
          _buildParam(
            {
              "soundPath": soundPath,
              "duration": duration,
            },
          ),
        ),
      ),
    );
  }

  /// 创建视频消息（视频最大支持 100 MB）
  ///
  /// 参数
  ///
  /// ```
  /// type	视频类型，如 mp4 mov 等
  /// duration	视频时长，单位 s
  /// snapshotPath	视频封面图片路径
  /// ```
  ///
  Future<V2TimValueCallback<String>> createVideoMessage({
    @required String videoFilePath,
    @required String type,
    @required int duration,
    @required String snapshotPath,
  }) async {
    return V2TimValueCallback<String>.fromJson(
      _formatJson(
        await _channel.invokeMethod(
          "createVideoMessage",
          _buildParam(
            {
              "videoFilePath": videoFilePath,
              "type": type,
            },
          ),
        ),
      ),
    );
  }

  /// 创建文件消息（文件最大支持 100 MB）
  ///
  Future<V2TimValueCallback<String>> createFileMessage({
    @required String filePath,
    @required String fileName,
  }) async {
    return V2TimValueCallback<String>.fromJson(
      _formatJson(
        await _channel.invokeMethod(
          "createFileMessage",
          _buildParam(
            {
              "filePath": filePath,
              "fileName": fileName,
            },
          ),
        ),
      ),
    );
  }

  /// 创建地理位置消息
  ///
  Future<V2TimValueCallback<String>> createLocationMessage({
    @required String desc,
    @required double longitude,
    @required double latitude,
  }) async {
    return V2TimValueCallback<String>.fromJson(
      _formatJson(
        await _channel.invokeMethod(
          "createLocationMessage",
          _buildParam(
            {
              "desc": desc,
              "longitude": longitude,
              "latitude": latitude,
            },
          ),
        ),
      ),
    );
  }

  /// 创建表情消息
  ///
  /// ```
  /// SDK 并不提供表情包，如果开发者有表情包，可使用 index 存储表情在表情包中的索引， 或者直接使用 data 存储表情二进制信息以及字符串 key，都由用户自定义，SDK 内部只做透传。
  /// ```
  ///
  /// 参数
  ///
  /// ```
  /// index	表情索引
  /// data	自定义数据
  /// ```
  ///
  Future<String> createFaceMessage({
    @required int index,
    @required String data,
  }) async {
    return await _channel.invokeMethod(
      "createFaceMessage",
      _buildParam(
        {
          "index": index,
          "data": data,
        },
      ),
    );
  }

  /// 获取单聊历史消息
  ///
  /// 参数
  ///
  /// ```
  /// count	拉取消息的个数，不宜太多，会影响消息拉取的速度，这里建议一次拉取 20 个
  /// lastMsg	获取消息的起始消息，如果传 null，起始消息为会话的最新消息
  /// ```
  ///
  /// 注意
  ///
  /// ```
  /// 如果 SDK 检测到没有网络，默认会直接返回本地数据
  /// ```
  ///
  Future<V2TimValueCallback<List<V2TimMessage>>> getC2CHistoryMessageList({
    @required String userID,
    @required int count,
    String lastMsgID,
  }) async {
    return V2TimValueCallback<List<V2TimMessage>>.fromJson(
      _formatJson(
        await _channel.invokeMethod(
          "getC2CHistoryMessageList",
          _buildParam(
            {
              "userID": userID,
              "count": count,
              "lastMsgID": lastMsgID,
            },
          ),
        ),
      ),
    );
  }

  /// 获取群组历史消息
  ///
  /// 参数
  ///
  /// ```
  /// count	拉取消息的个数，不宜太多，会影响消息拉取的速度，这里建议一次拉取 20 个
  /// lastMsg	获取消息的起始消息，如果传 null，起始消息为会话的最新消息
  /// ```
  ///
  /// 注意
  ///
  /// ```
  /// 如果 SDK 检测到没有网络，默认会直接返回本地数据
  /// 只有会议群（Meeting）才能拉取到进群前的历史消息，直播群（AVChatRoom）消息不存漫游和本地数据库，调用这个接口无效
  /// ```
  ///
  Future<V2TimValueCallback<List<V2TimMessage>>> getGroupHistoryMessageList({
    @required String groupID,
    @required int count,
    String lastMsgID,
  }) async {
    return V2TimValueCallback<List<V2TimMessage>>.fromJson(
      _formatJson(
        await _channel.invokeMethod(
          "getGroupHistoryMessageList",
          _buildParam(
            {
              "groupID": groupID,
              "count": count,
              "lastMsgID": lastMsgID,
            },
          ),
        ),
      ),
    );
  }

  /// 撤回消息
  ///
  /// 注意
  ///
  /// ```
  /// 撤回消息的时间限制默认 2 minutes，超过 2 minutes 的消息不能撤回，您也可以在 控制台（功能配置 -> 登录与消息 -> 消息撤回设置）自定义撤回时间限制。
  /// 仅支持单聊和群组中发送的普通消息，无法撤销 onlineUserOnly 为 true 即仅在线用户才能收到的消息，也无法撤销直播群（AVChatRoom）中的消息。
  /// 如果发送方撤回消息，已经收到消息的一方会收到 V2TIMAdvancedMsgListener -> onRecvMessageRevoked 回调。
  /// ```
  ///
  Future<V2TimCallback> revokeMessage({
    @required String msgID,
  }) async {
    return V2TimCallback.fromJson(
      _formatJson(
        await _channel.invokeMethod(
          "revokeMessage",
          _buildParam(
            {
              "msgID": msgID,
            },
          ),
        ),
      ),
    );
  }

  ///设置单聊消息已读
  ///
  Future<V2TimCallback> markC2CMessageAsRead({
    @required String userID,
  }) async {
    return V2TimCallback.fromJson(
      _formatJson(
        await _channel.invokeMethod(
          "markC2CMessageAsRead",
          _buildParam(
            {
              "userID": userID,
            },
          ),
        ),
      ),
    );
  }

  /// 设置群组消息已读
  ///
  Future<V2TimCallback> markGroupMessageAsRead({
    @required String groupID,
  }) async {
    return V2TimCallback.fromJson(
      _formatJson(
        await _channel.invokeMethod(
          "markGroupMessageAsRead",
          _buildParam(
            {
              "groupID": groupID,
            },
          ),
        ),
      ),
    );
  }

  /// 删除本地消息
  ///
  /// 注意
  ///
  /// ```
  /// 消息只能本地删除，消息删除后，SDK 会在本地把这条消息标记为已删除状态，getHistoryMessage 不能再拉取到， 但是如果程序卸载重装，本地会失去对这条消息的删除标记，getHistoryMessage 还能再拉取到该条消息。
  /// ```
  ///
  Future<V2TimCallback> deleteMessageFromLocalStorage({
    @required String msgID,
  }) async {
    return V2TimCallback.fromJson(
      _formatJson(
        await _channel.invokeMethod(
          "deleteMessageFromLocalStorage",
          _buildParam(
            {
              "msgID": msgID,
            },
          ),
        ),
      ),
    );
  }

  /// 删除本地及漫游消息
  ///
  /// 注意
  ///
  /// ```
  ///该接口会删除本地历史的同时也会把漫游消息即保存在服务器上的消息也删除，卸载重装后无法再拉取到。需要注意的是：
  ///   一次最多只能删除 30 条消息
  ///   要删除的消息必须属于同一会话
  ///   一秒钟最多只能调用一次该接口
  ///   如果该账号在其他设备上拉取过这些消息，那么调用该接口删除后，这些消息仍然会保存在那些设备上，即删除消息不支持多端同步。
  /// ```
  ///
  Future<V2TimCallback> deleteMessages({
    @required List<String> msgIDs,
  }) async {
    return V2TimCallback.fromJson(
      _formatJson(
        await _channel.invokeMethod(
          "deleteMessages",
          _buildParam(
            {
              "msgIDs": msgIDs,
            },
          ),
        ),
      ),
    );
  }

  Future<V2TimValueCallback<V2TimMessage>> insertGroupMessageToLocalStorage({
    @required String msgID,
    String groupID,
    @required String sender,
  }) async {
    return V2TimValueCallback<V2TimMessage>.fromJson(
      _formatJson(
        await _channel.invokeMethod(
          "insertGroupMessageToLocalStorage",
          _buildParam(
            {
              "msgID": msgID,
              "groupID": groupID,
              "sender": sender,
            },
          ),
        ),
      ),
    );
  }

  /// 发送高级消息（高级版本：可以指定优先级，推送信息等特性）
  ///
  /// 参数
  ///
  /// ```
  /// msg	待发送的消息对象，需要通过对应的 createXXXMessage 接口进行创建。
  /// receiver	消息接收者的 userID, 如果是发送 C2C 单聊消息，只需要指定 receiver 即可。
  /// groupID	目标群组 ID，如果是发送群聊消息，只需要指定 groupID 即可。
  /// priority	消息优先级，仅针对群聊消息有效。请把重要消息设置为高优先级（比如红包、礼物消息），高频且不重要的消息设置为低优先级（比如点赞消息）。
  /// onlineUserOnly	是否只有在线用户才能收到，如果设置为 true ，接收方历史消息拉取不到，常被用于实现“对方正在输入”或群组里的非重要提示等弱提示功能。
  /// offlinePushInfo	离线推送时携带的标题和内容。
  /// ```
  ///
  /// 返回
  ///
  /// ```
  /// 消息唯一标识
  /// ```
  ///
  /// 注意
  ///
  /// ```
  /// 设置 offlinePushInfo 字段，需要先在 V2TIMOfflinePushManager 开启推送。推送开启后，除了自定义消息，其他消息默认都会推送。
  /// 自定义消息默认不会推送，如果需要推送，请设置 offlinePushInfo 的 desc 字段，推送的时候会默认展示 desc 信息。
  /// ```
  ///
  Future<V2TimValueCallback<V2TimMessage>> sendMessage({
    String msgID,
    String receiver,
    String groupID,
    int priority,
    bool onlineUserOnly,
    Map offlinePushInfo,
  }) async {
    return V2TimValueCallback<V2TimMessage>.fromJson(
      _formatJson(
        await _channel.invokeMethod(
          "sendMessage",
          _buildParam(
            {
              "msgID": msgID,
              "groupID": groupID,
              "receiver": receiver,
              "priority": priority,
              "onlineUserOnly": onlineUserOnly,
              "offlinePushInfo": offlinePushInfo
            },
          ),
        ),
      ),
    );
  }

  ///@nodoc
  Map _buildParam(Map param) {
    param["TIMManagerName"] = "messageManager";
    return param;
  }

  ///@nodoc
  _formatJson(jsonSrc) {
    // print('json');
    // print(json.encode(jsonSrc));
    return json.decode(json.encode(jsonSrc));
  }
}
